/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package mv3302;

import simkit.random.RandomVariate;

/**
 * The ShipArrivalProcess class extends the ArrivalProcess class and represents
 * the arrival process for ships in a simulation. It generates ships at random
 * intervals and assigns random unloading times to each ship.
 *
 * @author dansl
 */
public final class ShipArrivalProcess extends ArrivalProcess {

    private RandomVariate interarrivalTimeGenerator;

    private RandomVariate unloadingTimeGenerator;

    /**
     * Constructs a new ShipArrivalProcess with no generators set.
     */
    public ShipArrivalProcess() {
    }

    /**
     * Constructs a new ShipArrivalProcess with the given generators for
     * interarrival times and unloading times.
     *
     * @param interarrivalTimeGenerator the generator for interarrival times
     * @param unloadingTimeGenerator the generator for unloading times
     */
    public ShipArrivalProcess(RandomVariate interarrivalTimeGenerator, RandomVariate unloadingTimeGenerator) {
        super(interarrivalTimeGenerator);
        setInterarrivalTimeGenerator(interarrivalTimeGenerator);
        setUnloadingTimeGenerator(unloadingTimeGenerator);
    }

    /**
     * Generates a new ship with a random unloading time and schedules its
     * arrival in the simulation.
     */
    @Override
    public void doArrival() {
        super.doArrival();
        Ship ship = new Ship(unloadingTimeGenerator.generate());
        waitDelay("ShipArrival", 0.0, ship);
    }

    /**
     * @return the unloadingTimeGenerator
     */
    public RandomVariate getUnloadingTimeGenerator() {
        return unloadingTimeGenerator;
    }

    /**
     * @param unloadingTimeGenerator the unloadingTimeGenerator to set
     */
    public void setUnloadingTimeGenerator(RandomVariate unloadingTimeGenerator) {
        this.unloadingTimeGenerator = unloadingTimeGenerator;
    }
}
